--- Recursively copies the given table, returning a new table with all existing keys and values
--- @generic T
--- @param table T The table to copy
--- @return T The copied table
local function copy(table)
  local newTable = --[[--- @type T ]] {}

  for k, v in pairs(--[[--- @type table<any, any> ]] table) do
    newTable[k] = type(v) == "table" and copy(--[[--- @type T ]] v) or v
  end

  return newTable
end

--- Merges two or more key, value pair tables into one. Note that this does not effect the input tables.
--- Note that precedence works in order of the parameters given, from lowest to highest.
--- @generic T
--- @param table1 T The first table to merge
--- @param table2 T the second table to merge
--- @vararg T Any other tables to merge
--- @return T The merge tables
local function merge(table1, table2, ...)
  local sourceTables = { table1, table2, ... }
  local mergedTable = --[[--- @type T ]] {}

  for _, sourceTable in ipairs(sourceTables) do
    for k, v in pairs(sourceTable) do
      mergedTable[k] = type(v) == "table" and merge(--[[--- @type T ]] v, mergedTable) or v
    end
  end

  return mergedTable
end

return {
  copy = copy,
  merge = merge
}