setExtensionUnloadMode("explosionsNG", "manual")

--- Creates a materials file for the level with the given path.
--- If the materials file has already been created, this function will not replace them.
--- @param levelPath string The level path to attempt to create materials for
local function createMaterialsForLevel(levelPath)
  local EXPECTED_PREFIX = "/levels"
  local MATERIAL_RELATIVE_PATH = "/explosionsNG/main.materials.json"
  -- smallgrid comes with this mod so we can copy from it
  local COPY_FROM = "levels/smallgrid"..MATERIAL_RELATIVE_PATH

  if not levelPath:find(EXPECTED_PREFIX) then
    log("W", "explosivesNG", "This level does not start with the expected prefix. This is either a bug in BeamNG or is a modded map. level="..tostring(levelPath))
    return
  end

  local copyTo = levelPath..MATERIAL_RELATIVE_PATH

  -- If the file already exists, we don't need to replace it
  if FS:fileExists(copyTo) then return end

  log("I", "explosivesNG", "Generating materials for "..levelPath)
  FS:copyFile(COPY_FROM, copyTo)
end

-- Check every level installed and generate the materials if needed
for _, levelPath in ipairs(core_levels.getLevelPaths()) do
  createMaterialsForLevel(levelPath)
end